#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Saxm2s - an xmms2 client, using python and QT
# Copyright (C) 2005  John O'Meara
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import sys, xmmsclient, math, os
from qt import *
import gettext

# Some globals. These are defined at build-time
SAX_BRANCH="@@@BZR_REVBRANCH@@@"
SAX_REVID= "@@@BZR_REVID@@@"
SAX_SHA1=  "@@@BZR_REVHASH@@@"
SAX_REVNO= "@@@BZR_REVNO@@@"

sys.path.insert(0,"@@@PYUIC_INST@@@")

# setup relocation paths
path=os.path.abspath(sys.argv[0])
path=os.path.dirname(path)
path=os.path.dirname(path)
sys.path.insert(0,"%s/share/saxm2s/app_module"%path)
gettext.install('saxm2s', localedir="%s/share/locale"%path, unicode=True)

import mainwin, media_widget, urldialog, mediaInfo

if (len(sys.argv) > 1 and sys.argv[1] == "--debug"):
	import pdb
	pdb.set_trace()

builtin_parm={
	"media_poll_delay": "100",
	"media_poll_at_once": "5",
	}

def mytr(text):
	"""mytr(text) -> QString
	calls _() to translate a string and converts to QString.
	This function is intended to be used as a replacement for tr()
	so that qt generated files can successfully use gettext.

	It should be installed into the builtin scope"""
	return QString(_(text))
import __builtin__
__builtin__.mytr=mytr #install mytr into the builtin scope
	
def secondsToMinutesandSeconds(sseconds):
	dmin = sseconds/60
	dsec = sseconds%60
	return str(dmin) + ':' + ('00' + str(dsec) )[-2:]

def printout(data):
	"""attempts to call print regularly,
	failing that, encode to ASCII replacing unknown characters.

	Hopefully this will provide the best method of output. If you know a better
	way, please contribute a patch!"""
	try:
		print data
		return
	except:
		pass
	try:
		lang=os.getenv('LANG')
		if lang and '.' in lang:
			ending = lang.split('.')[1]
			if '@' in ending:
				encoding = ending.split('@')[0]
			else:
				encoding = ending
			print data.encode(encoding)
			return
	except:
		pass
	print data.encode('ASCII','replace')
	
class playerUrlDLG(urldialog.UrlDialog):
	def __init__(self,parent=None):
		urldialog.UrlDialog__init__(self,parent)
		self.url=""
		self.ok=False
		self.connect(self.pushButtonAdd, SIGNAL('clicked()'), self.process)
	def process(self):
		self.url=str(self.lineEditURL.displayText())
		self.ok=True

class PlayListItem(QListViewItem):
	def compare(self, item, column, ascending):
		if column == 1:
			if self.key(1, ascending).toInt() < item.key(1, ascending).toInt():
				return -1
			else:
				return 1
		return 0

class saxconf:
	def __init__(self):
		self.file = xmmsclient.userconfdir_get() + "/clients/saxm2s.conf"
		self.parm=builtin_parm
		try:
			self.read(self.file)
		except IOError:
			printout (_("Warning: config file could not be opened: %s") % self.file)
			
	def read(self, file):
		conf = open(file)
		for line in conf:
			line=line.strip(' \r\n\t')
			if len(line) == 0 or line[0]=='#':
				continue
			print line
			(key,value) = line.split('=')
			self.parm[key]=value
		print self.parm
	def write(self, file=None):
		if file==None:
			file=self.file
		conf = open(file,'w')
		for (key,value) in self.parm.iteritems():
			conf.write("%s=%s\n" % key, value)

class playerWinDLG(mainwin.playerWin):
	def __init__(self,parent=None):
		mainwin.playerWin.__init__(self,parent)
		self.xmms = xmmsclient.XMMS()
		self.config = saxconf()
		xmmspath = os.getenv('XMMS_PATH')
		try:
			self.xmms.connect(xmmspath, self.hndDisconnect)
		except IOError:
			QMessageBox.critical(None, "Saxm2s", _("Failed to connect to a running xmms2 server. Start the server before starting SAXm2s"))
			sys.exit(1)
		self.songid = 0
		self.songtime = 0
		self.songpos = '-1'
		self.curtime = 0
		self.updateSlider = True
		self.updateSliderVolume = True
		self.updateSliderBalance = True
		self.polltimer = QTimer(self, "polltimer")
		self.playlistids = [0]
		self.xmms.broadcast_playlist_changed(self.cmdUpdatePlaylist)
		self.xmms.broadcast_medialib_entry_changed(self.hndMediaChange)
		self.xmms.broadcast_playlist_current_pos(self.hndPosChange)
		self.xmms.broadcast_playback_status(self.hndPlaybackStatusChange)
		self.xmms.broadcast_configval_changed(self.hndConfChange)
		self.xmms.broadcast_playback_volume_changed(self.hndPlaybackVolumeChange)
		#self.xmms.set_need_out_fun(self.io_out)
		self.fdMonitor = QSocketNotifier ( self.xmms.get_fd(), QSocketNotifier.Read)
		res = self.xmms.playback_volume_get()
		res.wait()
		if res.iserror():
			print "cannot show initial volume:",res.get_error()
		else:
			res=res.value()
			l=res['left']
			r=res['right']
			self.sliderVolume.setValue(max(l,r))
		
		self.loaditem=[] #should be a list of tuples, in the format of (QListViewItem,
		
		self.connect(self.polltimer, SIGNAL('timeout()'), self.checkstatus)
		self.connect(self.pushButtonBack, SIGNAL('clicked()'), self.cmdPrev)
		self.connect(self.pushButtonPlay, SIGNAL('clicked()'), self.cmdPlay)
		self.connect(self.pushButtonPause, SIGNAL('clicked()'), self.cmdPause)
		self.connect(self.pushButtonStop, SIGNAL('clicked()'), self.cmdStop)
		self.connect(self.pushButtonNext, SIGNAL('clicked()'), self.cmdNext)
		self.connect(self.slider1, SIGNAL('sliderPressed()'), self.sliderUpdateOff)
		self.connect(self.slider1, SIGNAL('sliderReleased()'), self.sliderUpdateOn)
		self.connect(self.slider1, SIGNAL('valueChanged(int)'), self.cmdSeek)
		self.connect(self.listViewPlayList, SIGNAL('doubleClicked(QListViewItem*)'), self.cmdJump)
		self.connect(self.listViewPlayList, SIGNAL('spacePressed(QListViewItem*)'), self.cmdJump)
		self.connect(self.pushButtonAdd, SIGNAL('clicked()'), self.cmdAddFile)
		self.connect(self.pushButtonAddU, SIGNAL('clicked()'), self.cmdAddUrl)
		self.connect(self.sliderVolume, SIGNAL('valueChanged(int)'), self.cmdVolume)
		self.connect(self.sliderBalance, SIGNAL('valueChanged(int)'), self.cmdBalance)
		self.connect(self.fdMonitor, SIGNAL('activated(int)'), self.io_in)
		self.connect(self.listViewPlayList, SIGNAL('contextMenuRequested(QListViewItem *, const QPoint &, int)'), self.cmenu)
		self.cmdUpdatePlaylist()
		
		sbar = self.statusBar()
		self.slabel = QLabel(self)
		sbar.addWidget(self.slabel,1,True)

		self.media=media_widget.media_widget(self.tabMain,"media")
		self.tabMain.insertTab(self.media,QString("media"))

		self.ConfPage = QWidget(self.tabMain,"ConfPage")
		ConfPageLayout = QVBoxLayout(self.ConfPage,11,6,"ConfPageLayout")
		
		ccol = QGridLayout(1,2)
		confwidgets = []
		for item in self.config.parm.keys():
			tmp=QLabel(self.ConfPage,item)
			tmp.setText(item)
			t2=QLineEdit(self.ConfPage,item)
			t2.setText(self.config.parm[item])
			t2.setEnabled(0)
			confwidgets.append((tmp,t2))
		counter=1
		for widget in confwidgets:
			ccol.addWidget(widget[0],counter,1)
			ccol.addWidget(widget[1],counter,2)
			counter += 1
		ConfPageLayout.addItem(ccol)
		ConfPageLayout.addItem(QSpacerItem(1,2,QSizePolicy.Expanding,QSizePolicy.Expanding))
        	self.tabMain.insertTab(self.ConfPage,QString("saxm2s"))

		self.hndPosChange() #this should update the info as far as title and slider go
		res=self.xmms.playback_status()
		res.wait()
		self.hndPlaybackStatusChange(res)
		self.listViewPlayList.setSorting(1, True) # sort by song number ascending
		self.xmms.signal_playback_playtime(self.updateTime)
		printout (_("done init"))
	def songmagnitude(self):
		if len(self.playlistids) == 0:
			return 0
		num=int(math.log(len(self.playlistids),10))
		return num
	def songprepend(self, num):
		prependamount = ""
		for i in range(0, num):
			prependamount = prependamount + '0'
		return prependamount
	def updateTime(self, event = None):
		#print "updateTime"
		if not event:
			print "no event in updateTime() !?!?!"
			return # I don't know what to do if this is empty
		#event.wait()
		if event.iserror():
			print "oops:",event.get_error()
			return
		res=event.value()
		res = res/1000
		if self.curtime == res:
			event.restart()
			return
		self.curtime = res
		if self.updateSlider:
			self.updateSlider=False
			self.slider1.setValue(self.curtime)
			self.updateSlider=True
		cmin = self.curtime/60
		csec = (self.curtime)%60
		tmin = self.songtime/60
		tsec = self.songtime%60
		self.textLabelTime.setText(str(cmin) + ':' + ('00' + str(csec) )[-2:] + '/' + str(tmin) + ':' + ('00' + str(tsec))[-2:])
		#print "    updateTime restart()"
		res = event.restart()
		#print "    updateTime done"
	# TODO: Rename this function. It only updates the playlist now
	def checkstatus(self):
		if len(self.loaditem) > 0:
			donow = []
			self.slabel.setText(_("updating playlist, %i remaining.") % len(self.loaditem))
			
			grabnum = int(self.config.parm['media_poll_at_once'])
			if len(self.loaditem) > grabnum:
				donow = self.loaditem[:grabnum]
				self.loaditem = self.loaditem[grabnum-1:]
				# there is stuff left over, make our delay less
				self.polltimer.stop()
				self.polltimer.start(int(self.config.parm['media_poll_delay']), False)
			else:
				donow = self.loaditem
				self.loaditem = []
				# there is nothing left to do after this run, return to normal speed
				self.polltimer.stop()
				self.slabel.setText(_("updating playlist, done."))
			for song in donow:
				songinfo = self.xmms.medialib_get_info(song[2])
				songinfo.wait()
				songinfo=songinfo.get_propdict()
				self.setItemsSongData(song[0],songinfo, song[1])
		if self.songid == 0:
			self.textLabelTime.setText("-:--/-:--")
			if self.updateSlider:
				self.slider1.setValue(0)
	def cmdNext(self):
		res=self.xmms.playlist_set_next_rel(1)
		res.wait()
		res=self.xmms.playback_tickle()
		res.wait()
	def cmdPrev(self):
		res=self.xmms.playlist_set_next_rel(-1)
		res.wait()
		res=self.xmms.playback_tickle()
		res.wait()
	def cmdPlay(self):
		res=self.xmms.playback_status() # 0=stop, 1=play, 2=pause
		res.wait()
		if(res.value() == 1):
			res=self.xmms.playback_tickle()
			res.wait()
		else:
			res=self.xmms.playback_start()
			res.wait()
	def cmdPause(self):
		res=self.xmms.playback_status()
		res.wait()
		if(res.value() == 0):
			return
		elif (res.value() == 2):
			res=self.xmms.playback_start()
			res.wait()
		else:
			res=self.xmms.playback_pause()
			res.wait()
	def cmdStop(self):
		self.songtime=0
		res=self.xmms.playback_stop()
		res.wait()
	def cmdJump(self,item):
		self.cmdStop()
		self.xmms.playlist_set_next(int(str(item.text(1)))-1)
		if self.updateSlider:
			self.slider1.setValue(0)
		res=self.xmms.playback_tickle()
		res.wait()
		self.cmdPlay()
	def cmdSeek(self, pos):
		#print "cmdSeek: ", pos
		if pos == self.curtime or self.songid == 0: # this was either called because we set it, or the user dropped it where we are. --or-- we shouldn't be updating, since there isn't really a song...
			#print "   seek ignored"
			return
		if not self.updateSlider: # Show seek to position in status bar
			self.slabel.setText(_("Seek to %s") % secondsToMinutesandSeconds(pos))
			return
		#print "    seeking to ", pos*1000
		ret=self.xmms.playback_seek_ms(pos*1000)
		ret.wait()
	def sliderUpdateOff(self):
		self.updateSlider=False
	def sliderUpdateOn(self):
		self.updateSlider=True
		self.cmdSeek(self.slider1.value())
	def cmdUpdatePlaylist(self, event=None):
		if (event != None):
			event = event.get_dict()
		#if event:
		#	print event.value()

		# XXX: now that there is a nice XMMSResult emmitted with nice data, if it is received
		# XXX:   something other than redoing the whole list should be done.
		# XXX:
		# XXX: The new data format is like this:
		# XXX: {'position': 30, 'type': 0, 'id': 2} # for new entry
		# XXX: {'type': 1} # for shuffle
		# XXX: {'newposition': 0, 'position': 28, 'type': 4, 'id': 11} # for moved entry
		# XXX: {'type': 5} # for sort, or more likely, reverse list
		# XXX: {'position': 52, 'type': 2} # for removed item
		
		# XXX: things seem to have changed just prior to the Dr, Alban release
		#      looks like this now:
		#      {'position': 962, 'type': 0, 'id': 1026}                add song
		#
		#      {'type': 2}                                             shuffle
		#      {'position': 950, 'type': 3}                            removed item 950
		#      {'type': 4}                                             clear
		#      {'newposition': 5, 'position': 2, 'type': 5, 'id': 590} move item 2 to position 5
		#      {'type': 6}                                             sort
		# currently the following constants are defined, make the code use them instead:
		# PLAYLIST_CHANGED_ADD = 0
		# PLAYLIST_CHANGED_CLEAR = 4
		# PLAYLIST_CHANGED_INSERT = 1
		# PLAYLIST_CHANGED_MOVE = 5
		# PLAYLIST_CHANGED_REMOVE = 3
		# PLAYLIST_CHANGED_SHUFFLE = 2
		# PLAYLIST_CHANGED_SORT = 6

		
		#print "Repopulating playlist"
		if event == None or event['type']==xmmsclient.PLAYLIST_CHANGED_SHUFFLE or event['type']==xmmsclient.PLAYLIST_CHANGED_SORT: # redo all
			list = self.xmms.playlist_list()
			list.wait()
			self.listViewPlayList.clear()
			counter = 1
			songlist=list.value()
			self.playlistids = songlist
			for song in songlist:
				#print songinfo
				item = PlayListItem(self.listViewPlayList, None)
				self.loaditem.append((item,str(counter),song))
				self.setItemsSongData(item,None, str(counter))
				counter = counter + 1
			#print "playlist repopulated with " + str(counter) + " entries"
		elif event['type']==xmmsclient.PLAYLIST_CHANGED_ADD: # new entry
			point=str(event['position']+1)
			item = PlayListItem(self.listViewPlayList, self.listViewPlayList.lastItem(), None)
			self.loaditem.append((item,point,event['id']))
			self.setItemsSongData(item,None, point)
			self.playlistids.append(event['id'])
			#self.setItemsSongData(item,songinfo, prependamount+str(event['positon'] + 1)[-(num+1):])
		elif event['type']==xmmsclient.PLAYLIST_CHANGED_REMOVE: # removed entry
			pos=event['position']
			#num=self.songmagnitude()
			#prepend=self.songprepend(num)
			ipos=pos
			#pos=(prepend + str(pos + 1))[-(num+1):]
			pos=str(pos + 1)
			item = self.listViewPlayList.findItem(pos,1)
			if (item == None):
				print "error! couldn't find song at " + pos + " for removal."
			else:
				next = item.nextSibling()
				last = self.listViewPlayList.lastItem()
				del self.playlistids[ipos]
				if not item == last: # there is stuff after, numbers need to be updated
					counter = ipos + 1 # add one since counter is for the visual, which is a 1 based list instead of 0
					while next != last:
						self.setItemsSongData(next,None,str(counter))
						counter += 1
						next = next.nextSibling()
					self.setItemsSongData(next,None,str(counter))
				self.listViewPlayList.takeItem(item)
				del item # I don't know if this is the correct way to get rid of an item...
				
		elif event['type']==xmmsclient.PLAYLIST_CHANGED_CLEAR: # clear list
			self.listViewPlayList.clear()
			self.playlistids=[]
		elif event['type']==xmmsclient.PLAYLIST_CHANGED_MOVE: # move an entry
			print "I don't support moving entries in the playlist yet"
			self.cmdUpdatePlaylist() # reload list as fallback
		else:
			print "unknown action requested of the playlist. event:"
			print event
			self.cmdUpdatePlaylist() # reload as best bet
			return
		self.polltimer.stop()
		self.polltimer.start(int(self.config.parm['media_poll_delay']), False)
		return # uncomment to prevent crashes on playlist loads
		# now select the current song and label it as current.
		pos=self.xmms.playlist_current_pos()
		pos.wait()
		pos=pos.value()
		if pos == None:
			return
		pos=int(pos) + 1
		num=self.songmagnitude()
		prepend=self.songprepend(num)
		pos=(prepend + str(pos))[-(num+1):]
		item = self.listViewPlayList.findItem(pos,1)
		if (item == None):
			print "error! couldn't find song " + pos
		else:
			item.setText(0,">")
			self.listViewPlayList.ensureItemVisible(item)
			self.songpos=pos
		
	def cmdAddFile(self):
		bob=QFileDialog("","Audio Files (*.ogg *.mp3 *.wav *.flac);;Ogg Vorbis (*.ogg);;All Files (*)")
		bob.setMode(QFileDialog.ExistingFiles)
		bob.exec_loop()
		for i in bob.selectedFiles():
			res=self.xmms.playlist_add("file://" + str(i))
			#print "file:///" + str(i)
			res.wait()
	def cmdAddUrl(self):
		urlwin=playerUrlDLG(parent=self)
		urlwin.show()
		urlwin.exec_loop()
		if urlwin.ok:
			self.xmms.playlist_add(urlwin.url).wait()
	def cmdVolume(self, newvol):
		#calculate the current balance
		if not self.updateSliderVolume: # don't change anything
			return
		res=self.xmms.playback_volume_get()
		res.wait()
		res=res.value()
		(l,r)=res['left'],res['right']
		newl=0
		newr=0
		if l==r:
			newl=newvol
			newr=newvol
		elif l<r:
			perc=l*100/r
			newl=(newvol*perc/100)
			newr=newvol
		else:
			perc=r*100/l
			newl=newvol
			newr=(newvol*perc/100)
		
		res=self.xmms.playback_volume_set('left', newl)
		res2=self.xmms.playback_volume_set('right', newr)
		res.wait()
		res2.wait()
	def cmdBalance(self, newbal):
		if not self.updateSliderBalance: # don't change anything
			return
		res=self.xmms.playback_volume_get()
		res.wait()
		res=res.value()
		(l,r)=res['left'],res['right']
		vol=max(l,r)
		if newbal == 50:
			newl = vol
			newr = vol
			self.textLabelBalPerc.setNum(0)
			self.textLabelBalDir.setText("%C")
		elif newbal > 50: # right side is louder
			newbal = (100-newbal)*2 # 50-(newbal-50), invert and get it between 0 and 50
			newl = vol*newbal/100
			newr = vol
			self.textLabelBalPerc.setNum(100-newbal)
			self.textLabelBalDir.setText("%R")
		else:
			newbal = newbal*2
			newl = vol
			newr = vol*newbal/100
			self.textLabelBalPerc.setNum(100-newbal)
			self.textLabelBalDir.setText("%L")
		res=self.xmms.playback_volume_set('left', newl)
		res2=self.xmms.playback_volume_set('right', newr)
		res.wait()
		res2.wait()
		
	def hndMediaChange(self, event=None):
		changedid = event.value()
#		print "song with medialib id " + str(changedid) + " has had something change"
		songinfo = self.xmms.medialib_get_info(changedid)
		songinfo.wait()
		songinfo=songinfo.get_propdict()
#		print "\tsearching for positions with the song..."
#		print "\t is " + str(changedid) + " in ",
#		print self.playlistids
		occurances = self.playlistids.count(changedid)
		if occurances == 0: # if its not in our playlist, we don't care
#			print "\tnot found"
			return
		index=0
		for i in range(0,occurances):
			index = self.playlistids.index(changedid,index)
#			print "\t found at " + str(index)
			pos=str(index+1)
			item = self.listViewPlayList.findItem(pos,1)
			if item == None:
				print "glitch! should have found item at playlist index " + str(index+1) + " but we didn't!"
			else:
				self.setItemsSongData(item,songinfo, pos)
		if self.songid == songinfo['id']:
			self.updateTitleAndBar(songinfo)
	def setItemsSongData(self,item,songdata, id):
		item.setText(1, str(id))
		if (songdata == None):
			return
		if songdata.has_key('title'):
			item.setText(2,songdata['title'])
		if songdata.has_key('artist'):
			item.setText(3,songdata['artist'])
		if songdata.has_key('duration'):
			item.setText(4,secondsToMinutesandSeconds(int(songdata['duration'])/1000))
		else:
			item.setText(4,"-:--")
	def hndPosChange(self, event=None): # called when a new entry has started to be played. Update list to show current song, and fix slider pos.
		if not event:
			event=self.xmms.playlist_current_pos()
			event.wait()
		newpos=event.value()
		if newpos == None: # we aren't playing anything. This is only likely to be reached at startup
			self.updateTitleAndBar()
			return
		res=self.xmms.playlist_list()
		res.wait()
		songlist=res.value()
		songid=songlist[newpos]
		if songid != self.songid:
			self.songid = songid
			res=self.xmms.medialib_get_info(self.songid)
			res.wait()
			songdata=res.get_propdict()
			if (songdata[('server','id')] == '0'):
				pass # no current song
			else:
				self.updateTitleAndBar(songdata)
				self.updateSlider = False
				self.slider1.setMaxValue(self.songtime)
				self.updateSlider = True
				pos=newpos + 1
				pos=str(pos)
				item = self.listViewPlayList.findItem(self.songpos,1)
				if (item != None):
					item.setText(0, "")
				item = self.listViewPlayList.findItem(pos,1)
				if (item == None):
					print "error! couldn't find song " + pos
				else:
					item.setText(0,">")
					self.songpos=pos
					self.listViewPlayList.ensureItemVisible(item)
				# XXX: this should disappear when the MediaInfo tab goes away
				info=""
				for i,j in songdata.iteritems():
					info = info + (unicode(i) + " = " + unicode(j)) + "\n"
				self.textBrowserMediaInfo.setText(info)
				# XXX: end of MediaInfo tab updater
	def updateTitleAndBar(self, songdata=None):
		if not songdata:
			if len(self.playlistids) == 0:
				return
			res=self.xmms.medialib_get_info(self.songid)
			res.wait()
			try:
				songdata=res.get_dict()
			except ValueError:
				print "failed to get song information! - skipping"
				print "song id was: ", self.songid
				print "Probably no song loaded"
				return
		if songdata.has_key('title'):
			self.textLabelTitle.setText(songdata['title'])
			self.setCaption("%s - xmms2" % songdata['title'])
		else:
			self.textLabelTitle.setText("???")
			self.setCaption("xmms2")
		if songdata.has_key('artist'):
			self.textLabelArtist.setText(songdata['artist'])
		else:
			self.textLabelArtist.setText("???")
		if songdata.has_key('duration'):
			self.songtime = int(songdata['duration']) / 1000
			self.slider1.show()
		else:
			self.songtime = 0
			self.slider1.hide()
	def hndPlaybackStatusChange(self, event=None):
		if not event:
			return # what else to do?
		res=event.get_uint()
		statetext=""
		if (res == xmmsclient.PLAYBACK_STATUS_STOP):
			statetext = _("Stopped")
			self.textBrowserMediaInfo.setText("")
		elif (res == xmmsclient.PLAYBACK_STATUS_PLAY):
			statetext = _("Playing")
		elif (res == xmmsclient.PLAYBACK_STATUS_PAUSE):
			statetext = _("Paused")
		else:
			statetext = _("<strong>unknown!</strong>")
		self.textLabelState.setText(statetext)
	def hndDisconnect(self, event=None):
		QMessageBox.critical(None, "Saxm2s", _("Lost connection with server. Exiting."))
		self.close()
	def hndConfChange(self, event=None):
		res = event.value()
		if res['name'] == 'output.volume':
			vol=res['value']
			(l,r)=vol.split('/')
			l=int(l)
			r=int(r)
			if not self.sliderVolume.value() == max(l,r):
				self.updateSliderVolume = False
				self.sliderVolume.setValue(max(l,r))
				self.updateSliderVolume = True
			if l > r:
				balance = r*50/l
			elif r>l:
				balance = 100 - (l*50/r)
			else:
				balance = 50
			if not self.sliderBalance.value() == balance:
				print "balance update to",balance
				self.updateSliderBalance = False
				self.sliderBalance.setValue(balance)
				self.updateSliderBalance = True
	def hndPlaybackVolumeChange(self, event = None):
		vol = event.value()
		l,r = vol['left'], vol['right']
		
		self.updateSliderVolume = False
		self.sliderVolume.setValue(max(l,r))
		self.updateSliderVolume = True

		if l>r:
			balance = r*50/l
		elif r>l:
			balance = 100 - (l*50/r)
		else:
			balance = 50
		self.updateSliderBalance = False
		self.sliderBalance.setValue(balance)
		self.updateSliderBalance = True

		# Why is this needed? aren't I already the handler for this?
		self.xmms.broadcast_playback_volume_changed(self.hndPlaybackVolumeChange)
	def io_in(self, tmp=None):
#		print "IO_in"
		self.xmms.ioin() #force check for inbound data.
		if self.xmms.want_ioout():
			self.xmms.ioout()
#		print "endIO_in"
	def io_out(self, tmp=None):
		print "IO_out"
		self.xmms.ioout()
		print "endIO_out"
	def cmenu(self, item, point, col):
		print "cmenu called"
		men=QPopupMenu(self)
		remitem=lambda : self.delentry(item) # make a function we can give to QT which will send it the item we want. Yay lambda!
		men.insertItem(_("remove item"),remitem)
		popupinfo=lambda : self.entryPopup(item)
		men.insertItem(_("Song Information"),popupinfo)
		men.exec_loop(point)
	def delentry(self,item=None):
		print "delentry called. data=",item
		if item:
			id = int(str(item.text(1)))-1
			print "remove songid:",id
			res=self.xmms.playlist_remove(id)
			res.wait()
			#self.listViewPlayList.takeItem(item)
			#del item
	def entryPopup(self, item=None):
		if item:
			id = int(str(item.text(1)))-1
			songid = self.playlistids[id]
			self.showMediaInfo(songid)
	def showMediaInfo(self, Id):
		popup = mediaInfo.mediaInfo(self)
		# give basic info
		res = self.xmms.medialib_get_info(Id)
		res.wait()
		songdata = res.get_propdict()
		 # First, a helper function for ourselves
		def set(widget, key):
			if songdata.has_key(key):
				widget.setText(unicode(songdata[key]))
			else:
				widget.setText(_("unknown"))
				widget.setDisabled(1)
		 # Second, use the helper a lot!
		set(popup.simple_title,'title')
		set(popup.simple_artist,'artist')
		set(popup.simple_genre,'genre')
		set(popup.simple_duration,'duration')
		set(popup.simple_album,'album')
		set(popup.simple_playcount,'timesplayed')
		info=""
		for i,j in songdata.iteritems():
			info = info + (unicode(i) + _(" = ") + unicode(j)) + "\n"
		popup.extended.setText(info)
		popup.show()
		
		# give all info


if __name__== '__main__':
	app = QApplication(sys.argv)
	QObject.connect(app, SIGNAL('lastWindowClosed()'), app, SLOT('quit()'))
	win = playerWinDLG()
	app.setMainWidget(win)
	win.show()
	app.exec_loop()


#>>> temp.get_hashtable()
#{'resolved': '1', 'album': 'http://www.ocremix.org', 'added': '1116714251', 'laststarted': '1116723681', 'title': 'Space Harrier I got the Spacebeam OC ReMix', 'url': 'file:///home/john/media/Space_Harrier_I_Got_the_Spacebeam_OC_ReMix.mp3', 'bitrate': '160', 'artist': 'Star Salzman', 'date': '2004', 'mime': 'audio/mpeg', 'duration': '176128', 'genre': 'Game', 'lmod': '1116151709', 'id': '2'}
